@@ -341,49 +341,69 @@ int libxmp_snprintf (char *, size_t, const char *, ...) LIBXMP_ATTRIB_PRINTF(3,4
 #define FLOW_LOOP_FIRST_EFFECT	(1 <<  6) /* Only execute the first E60/E6x in a row */
 #define FLOW_LOOP_ONE_AT_A_TIME	(1 <<  7) /* Init E6x if no other channel is looping (MPT) */
 #define FLOW_LOOP_IGNORE_TARGET	(1 <<  8) /* Ignore E60 if count is >=1 (LIQ) */
+#define FLOW_LOOP_DELAY_BREAK	(1 <<  9) /* E6x jump prevents later Dxx on same row (S3M, IT) */
+#define FLOW_LOOP_DELAY_JUMP	(1 << 10) /* E6x jump prevents later Bxx on same row (S3M) */
+#define FLOW_LOOP_UNSET_BREAK	(1 << 11) /* E6x jump cancels prior Dxx on same row (S3M, IMF) */
+#define FLOW_LOOP_UNSET_JUMP	(1 << 12) /* E6x jump cancels prior Bxx on same row (S3M) */
+#define FLOW_LOOP_SHARED_BREAK	(1 << 13) /* E6x overrides prior Dxx dest on same row (LIQ) */
 /*#define FLOW_LOOP_TICK_0_JUMP */	  /* Loop jump shortens row to one tick (DTM) */
 
 #define HAS_FLOW_MODE(x)	(m->flow_mode & (x))
 
 #define FLOW_MODE_GENERIC	0
 #define FLOW_LOOP_GLOBAL	(FLOW_LOOP_GLOBAL_TARGET | FLOW_LOOP_GLOBAL_COUNT)
+/* Simulate players where all breaks and jumps are ignored during a loop jump. */
+#define FLOW_LOOP_NO_BREAK_JUMP	(FLOW_LOOP_DELAY_BREAK | FLOW_LOOP_DELAY_JUMP | \
+				 FLOW_LOOP_UNSET_BREAK | FLOW_LOOP_UNSET_JUMP)
 
 /* Scream Tracker 3. No S3Ms seem to rely on the earlier behavior mode.
  * 3.01b has a bug where the end advancement sets the target to the same line
  * instead of the next line; there's no way to make use of this without getting
  * stuck, so it's not simulated.
  */
-#define FLOW_MODE_ST3_301	(FLOW_MODE_ST3_321 | FLOW_LOOP_INIT_SAMEROW)
+#define FLOW_MODE_ST3_301	(FLOW_LOOP_GLOBAL | FLOW_LOOP_PATTERN_RESET | \
+				 FLOW_LOOP_END_ADVANCES | FLOW_LOOP_INIT_SAMEROW)
 #define FLOW_MODE_ST3_321	(FLOW_LOOP_GLOBAL | FLOW_LOOP_PATTERN_RESET | \
-				 FLOW_LOOP_END_ADVANCES)
+				 FLOW_LOOP_END_ADVANCES | FLOW_LOOP_NO_BREAK_JUMP)
 
-/* Impulse Tracker. Not clear if anything relies on either old behavior type.
+/* Impulse Tracker. Not clear if anything relies on the old behavior types.
+ * IT loops were global pre-1.04, and loop jumps override any prior break/jump.
+ * IT 2.00+ loop jumps also delay any following break, but not pattern jumps.
+ * IT 2.10+ reintroduced ST3's loop target advancement.
  */
-#define FLOW_MODE_IT_100	(FLOW_LOOP_GLOBAL)
-#define FLOW_MODE_IT_104	(FLOW_MODE_GENERIC)
-#define FLOW_MODE_IT_210	(FLOW_LOOP_END_ADVANCES)
+#define FLOW_MODE_IT_100	(FLOW_LOOP_GLOBAL | \
+				 FLOW_LOOP_UNSET_BREAK | FLOW_LOOP_UNSET_JUMP)
+#define FLOW_MODE_IT_104	(FLOW_LOOP_UNSET_BREAK | FLOW_LOOP_UNSET_JUMP)
+#define FLOW_MODE_IT_200	(FLOW_MODE_IT_104 | FLOW_LOOP_DELAY_BREAK)
+#define FLOW_MODE_IT_210	(FLOW_MODE_IT_200 | FLOW_LOOP_END_ADVANCES)
 
-/* Modplug Tracker/OpenMPT */
-#define FLOW_MODE_MPT_116	(FLOW_LOOP_ONE_AT_A_TIME)
+/* Modplug Tracker/early OpenMPT */
+#define FLOW_MODE_MPT_116	(FLOW_LOOP_ONE_AT_A_TIME | FLOW_LOOP_NO_BREAK_JUMP)
 
 /* Imago Orpheus. Pattern Jump actually does not reset target/count, but all
  * other forms of pattern change do. Unclear if anything relies on it.
+ * An XAx jump will set the destination row of a prior Txx jump.
+ * An XAx jump will cancel a prior Uxx break on the same row.
  */
-#define FLOW_MODE_ORPHEUS	(FLOW_LOOP_PATTERN_RESET)
+#define FLOW_MODE_ORPHEUS	(FLOW_LOOP_PATTERN_RESET | \
+				 FLOW_LOOP_SHARED_BREAK | FLOW_LOOP_UNSET_BREAK)
 
 /* Liquid Tracker uses generic MOD loops with an added behavior where
  * the end of a loop will cancel any other jump in the row that preceded it.
  * M60 is also ignored in channels that have started a loop for some reason.
+ * When M6x jumps, it overrides any prior break line set by Jxx/Cxx.
  * There is also a "Scream Tracker" compatibility mode (only detectable in the
  * newer format) that adds LOOP_MODE_PATTERN_RESET.
  */
-#define FLOW_MODE_LIQUID	(FLOW_LOOP_END_CANCELS | FLOW_LOOP_IGNORE_TARGET)
+#define FLOW_MODE_LIQUID	(FLOW_LOOP_END_CANCELS | FLOW_LOOP_IGNORE_TARGET | \
+				 FLOW_LOOP_SHARED_BREAK)
 #define FLOW_MODE_LIQUID_COMPAT	(FLOW_MODE_LIQUID | FLOW_LOOP_PATTERN_RESET)
 
 /* Octalyser (Atari). Looping jumps to the original position E60 was used in,
  * which libxmp doesn't simulate for now since it mostly gets the player stuck.
  * Octalyser ignores E60 if a loop is currently active; it's not clear if it's
- * possible for a module to actually rely on this behavior.
+ * possible for a module to actually rely on this behavior. Loop jumps
+ * interrupt all breaks/jumps on the same row.
  *
  * LOOP_MODE_END_CANCELS is inaccurate but needed to fix "Dammed Illusion",
  * which has multiple E6x on one line that don't trigger because the module
@@ -392,15 +412,22 @@ int libxmp_snprintf (char *, size_t, const char *, ...) LIBXMP_ATTRIB_PRINTF(3,4
  * because multiple E6x on a row otherwise traps the player.
  */
 #define FLOW_MODE_OCTALYSER	(FLOW_LOOP_GLOBAL | FLOW_LOOP_IGNORE_TARGET | \
-				 FLOW_LOOP_END_CANCELS)
+				 FLOW_LOOP_END_CANCELS | FLOW_LOOP_NO_BREAK_JUMP)
 
 /* Digital Tracker prior to shareware 1.02 doesn't use LOOP_MODE_FIRST_EFFECT,
  * but any MOD that would rely on it is impossible to fingerprint.
+ * Early versions had fully working loop jump precedence over jump/break;
+ * later versions gradually break this in ways libxmp only partially implements.
  * Commercial version 1.9(?) added per-track counters.
  * Digital Home Studio added a bizarre tick-0 jump bug.
  */
-#define FLOW_MODE_DTM_203	(FLOW_LOOP_GLOBAL | FLOW_LOOP_FIRST_EFFECT)
-#define FLOW_MODE_DTM_19	(FLOW_LOOP_GLOBAL_TARGET)
+#define FLOW_MODE_DTM_2015	(FLOW_LOOP_GLOBAL | FLOW_LOOP_FIRST_EFFECT | \
+				 FLOW_LOOP_NO_BREAK_JUMP)
+#define FLOW_MODE_DTM_2015_6CH	(FLOW_LOOP_GLOBAL | FLOW_LOOP_FIRST_EFFECT | \
+				 FLOW_LOOP_DELAY_BREAK | FLOW_LOOP_UNSET_BREAK | \
+				 FLOW_LOOP_SHARED_BREAK)
+#define FLOW_MODE_DTM_19	(FLOW_LOOP_GLOBAL_TARGET | FLOW_LOOP_UNSET_BREAK | \
+				 FLOW_LOOP_SHARED_BREAK)
 #define FLOW_MODE_DTM_DHS	(FLOW_LOOP_GLOBAL_TARGET | FLOW_LOOP_TICK_0_JUMP)
 
 
@@ -416,12 +443,16 @@ int libxmp_snprintf (char *, size_t, const char *, ...) LIBXMP_ATTRIB_PRINTF(3,4
 #define MED_TIME_FACTOR		2.64
 #define FAR_TIME_FACTOR		4.01373	/* See far_extras.c */
 
+#define NO_SEQUENCE		MAX_SEQUENCES
 #define MAX_SEQUENCES		255
 #define MAX_SAMPLE_SIZE		0x10000000
 #define MAX_SAMPLES		1024
 #define MAX_INSTRUMENTS		255
 #define MAX_PATTERNS		256
 
+#define XMP_MARK_SKIP		0xfe /* S3M/IT (QUIRK_MARKER) skip position */
+#define XMP_MARK_END		0xff /* S3M/IT (QUIRK_MARKER) end position */
+
 #define IS_PLAYER_MODE_MOD()	(m->read_event_type == READ_EVENT_MOD)
 #define IS_PLAYER_MODE_FT2()	(m->read_event_type == READ_EVENT_FT2)
 #define IS_PLAYER_MODE_ST3()	(m->read_event_type == READ_EVENT_ST3)
@@ -567,8 +598,8 @@ struct player_data {
 	int player_flags;
 	int flags;
 
-	double current_time;
-	double frame_time;
+	double scan_time_factor;	/* m->time_factor for most recent scan */
+	double current_time;		/* current time based on scan time factor */
 
 	int loop_count;
 	int sequence;
