@@ -45,10 +45,9 @@
 #define RETVAL_NOT_BZIP_DATA     (-1)
 #define RETVAL_DATA_ERROR        (-2)
 #define RETVAL_OBSOLETE_INPUT    (-3)
-// libxmp additions:
-#define RETVAL_OUT_OF_MEMORY         (-4)
-#define RETVAL_UNEXPECTED_INPUT_EOF  (-5)
-#define RETVAL_UNEXPECTED_OUTPUT_EOF (-6)
+#define RETVAL_EOF_IN            (-4)
+#define RETVAL_EOF_OUT           (-5)
+#define RETVAL_OUT_OF_MEMORY     (-6)   /* libxmp addition */
 
 // This is what we know about each huffman coding group
 struct group_data {
@@ -69,6 +68,10 @@ struct bwdata {
   unsigned int *dbuf;
 };
 
+#if defined(_MSC_VER)
+#pragma warning(push)
+#pragma warning(disable:4324) /* structure was padded due to alignment specifier */
+#endif
 // Structure holding all the housekeeping data, including IO buffers and
 // memory that persists between calls to bunzip
 struct bunzip_data {
@@ -100,8 +103,11 @@ struct bunzip_data {
   /* libxmp: For I/O error handling */
   jmp_buf jmpbuf;
 };
+#if defined(_MSC_VER)
+#pragma warning (pop)
+#endif
 
-// libxmp addition
+/* libxmp addition */
 struct bunzip_output {
   unsigned char *buf;
   size_t buf_size;
@@ -135,7 +141,7 @@ static unsigned int get_bits(struct bunzip_data *bd, char bits_wanted)
     // If we need to read more data from file into byte buffer, do so
     if (bd->inbufPos == bd->inbufCount) {
       if (0 >= (bd->inbufCount = hio_read(bd->inbuf, 1, IOBUF_SIZE, bd->in_fd))) /* libxmp: read -> hio_read */
-        longjmp(bd->jmpbuf, RETVAL_UNEXPECTED_INPUT_EOF); /* libxmp: error_exit -> longjmp */
+        longjmp(bd->jmpbuf, RETVAL_EOF_IN);
       bd->inbufPos = 0;
     }
 
@@ -469,11 +475,11 @@ static int flush_bunzip_outbuf(struct bunzip_data *bd, struct bunzip_output *out
     if (bd->outbufPos > out_fd->buf_alloc - out_fd->buf_size) {
       size_t new_size = next_power_of_two_32(out_fd->buf_size + bd->outbufPos);
       if (new_size <= out_fd->buf_alloc || new_size > LIBXMP_DEPACK_LIMIT)
-        return RETVAL_UNEXPECTED_OUTPUT_EOF;
+        return RETVAL_EOF_OUT;
 
       buf = (unsigned char *)realloc(buf, new_size);
       if (!buf)
-        return RETVAL_UNEXPECTED_OUTPUT_EOF;
+        return RETVAL_EOF_OUT;
 
       out_fd->buf = buf;
       out_fd->buf_alloc = new_size;
@@ -485,7 +491,7 @@ static int flush_bunzip_outbuf(struct bunzip_data *bd, struct bunzip_output *out
   return 0;
 }
 
-static void burrows_wheeler_prep(struct bunzip_data *bd, struct bwdata *bw)
+static void burrows_wheeler_prep(struct bwdata *bw)
 {
   int ii, jj;
   unsigned int *dbuf = bw->dbuf;
@@ -532,7 +538,7 @@ static int read_bunzip_data(struct bunzip_data *bd)
   if (!rc) rc=read_huffman_data(bd, bd->bwdata);
 
   // First thing that can be done by a background thread.
-  burrows_wheeler_prep(bd, bd->bwdata);
+  burrows_wheeler_prep(bd->bwdata);
 
   return rc;
 }
@@ -547,15 +553,18 @@ static int read_bunzip_data(struct bunzip_data *bd)
 
 /* libxmp: int -> struct bunzip_output * */
 static int write_bunzip_data(struct bunzip_data *bd, struct bwdata *bw,
-  struct bunzip_output *out_fd, char *outbuf, int len)
+  struct bunzip_output *out_fd, char *outbuf, const int len_)
 {
   unsigned int *dbuf = bw->dbuf;
-  int count, pos, current, run, copies, outbyte, previous, gotcount = 0;
+  int count, pos, current, run, copies, outbyte, previous, gotcount, len;
 
   /* libxmp: Reset longjmp I/O error handling */
   int ret = setjmp(bd->jmpbuf);
   if (ret) return ret;
 
+  gotcount = 0;
+  len = len_;
+
   for (;;) {
     // If last read was short due to end of file, return last block now
     if (bw->writeCount < 0) return bw->writeCount;
@@ -738,5 +747,6 @@ static int decrunch_bzip2(HIO_HANDLE *in, void **out, long *outlen)
 
 const struct depacker libxmp_depacker_bzip2 = {
 	test_bzip2,
+	NULL,
 	decrunch_bzip2
 };
