@@ -36,16 +36,16 @@ struct hmn_module_extras {
 	(HMN_MODULE_EXTRAS(x) != NULL && \
 	 HMN_MODULE_EXTRAS(x)->magic == HMN_EXTRAS_MAGIC)
 
-void hmn_play_extras(struct context_data *, struct channel_data *, int);
-void hmn_set_arpeggio(struct channel_data *, int);
-int hmn_linear_bend(struct context_data *, struct channel_data *);
-int hmn_new_instrument_extras(struct xmp_instrument *);
-int hmn_new_channel_extras(struct channel_data *);
-void hmn_reset_channel_extras(struct channel_data *);
-void hmn_release_channel_extras(struct channel_data *);
-int hmn_new_module_extras(struct module_data *);
-void hmn_release_module_extras(struct module_data *);
-void hmn_extras_process_fx(struct context_data *, struct channel_data *, int, uint8, uint8, uint8, int);
+void libxmp_hmn_play_extras(struct context_data *, struct channel_data *, int);
+void libxmp_hmn_set_arpeggio(struct channel_data *, int);
+int  libxmp_hmn_linear_bend(struct context_data *, struct channel_data *);
+int  libxmp_hmn_new_instrument_extras(struct xmp_instrument *);
+int  libxmp_hmn_new_channel_extras(struct channel_data *);
+void libxmp_hmn_reset_channel_extras(struct channel_data *);
+void libxmp_hmn_release_channel_extras(struct channel_data *);
+int  libxmp_hmn_new_module_extras(struct module_data *);
+void libxmp_hmn_release_module_extras(struct module_data *);
+void libxmp_hmn_extras_process_fx(struct context_data *, struct channel_data *, int, uint8, uint8, uint8, int);
 
 #endif
 
