@@ -22,7 +22,7 @@
 
 #include "loader.h"
 #include "mod.h"
-#include "period.h"
+#include "../period.h"
 
 static int chip_test(HIO_HANDLE *, char *, const int);
 static int chip_load(struct module_data *, HIO_HANDLE *, const int);
@@ -60,7 +60,8 @@ static int chip_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	LOAD_INIT();
 
-	if ((tidx = calloc(1, 1024)) == NULL) {
+	tidx = (uint8 *) calloc(1, 1024);
+	if (tidx == NULL) {
 		goto err;
 	}
 
