@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2022 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 
 #include "loader.h"
 #include "iff.h"
-#include "period.h"
+#include "../period.h"
 
 #define MAGIC_DMDL	MAGIC4('D','M','D','L')
 
@@ -99,37 +99,37 @@ struct local_data {
 
 
 static void fix_env(int i, struct xmp_envelope *ei, struct mdl_envelope *env,
-		    int *index, int envnum)
+		    int *idx, int envnum)
 {
     int j, k, lastx;
 
-    if (index[i] >= 0) {
+    if (idx[i] >= 0) {
 	ei->flg = XMP_ENVELOPE_ON;
 	ei->npt = 15;
 
 	for (j = 0; j < envnum; j++) {
-    	    if (index[i] == env[j].num) {
-    	        ei->flg |= env[j].sus & 0x10 ? XMP_ENVELOPE_SUS : 0;
-    	        ei->flg |= env[j].sus & 0x20 ? XMP_ENVELOPE_LOOP : 0;
-    	        ei->sus = env[j].sus & 0x0f;
-    	        ei->lps = env[j].loop & 0x0f;
-    	        ei->lpe = env[j].loop & 0xf0;
+	    if (idx[i] == env[j].num) {
+	        ei->flg |= env[j].sus & 0x10 ? XMP_ENVELOPE_SUS : 0;
+	        ei->flg |= env[j].sus & 0x20 ? XMP_ENVELOPE_LOOP : 0;
+	        ei->sus = env[j].sus & 0x0f;
+	        ei->lps = env[j].loop & 0x0f;
+	        ei->lpe = env[j].loop & 0xf0;
 
 		lastx = -1;
 
-    	        for (k = 0; k < ei->npt; k++) {
+		for (k = 0; k < ei->npt; k++) {
 		    int x = env[j].data[k * 2];
 
-        	    if (x == 0)
-        		    break;
-        	    ei->data[k * 2] = lastx + x;
-        	    ei->data[k * 2 + 1] = env[j].data[k * 2 + 1];
+		    if (x == 0)
+			break;
+		    ei->data[k * 2] = lastx + x;
+		    ei->data[k * 2 + 1] = env[j].data[k * 2 + 1];
 
 		    lastx = ei->data[k * 2];
-        	}
+		}
 
-        	ei->npt = k;
-        	break;
+		ei->npt = k;
+		break;
             }
         }
     }
@@ -379,6 +379,7 @@ static int get_chunk_in(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->has_in) {
+	D_(D_CRIT "duplicate IN chunk");
 	return -1;
     }
     data->has_in = 1;
@@ -426,13 +427,15 @@ static int get_chunk_pa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->has_pa || !data->has_in) {
+	D_(D_CRIT "duplicate PA chunk or missing IN chunk");
 	return -1;
     }
     data->has_pa = 1;
 
     mod->pat = hio_read8(f);
 
-    if ((mod->xxp = calloc(sizeof (struct xmp_pattern *), mod->pat)) == NULL)
+    mod->xxp = (struct xmp_pattern **) calloc(mod->pat, sizeof(struct xmp_pattern *));
+    if (mod->xxp == NULL)
         return -1;
 
     D_(D_INFO "Stored patterns: %d", mod->pat);
@@ -465,13 +468,15 @@ static int get_chunk_p0(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->has_pa || !data->has_in) {
+	D_(D_CRIT "duplicate PA (0.0) chunk or missing IN chunk");
 	return -1;
     }
     data->has_pa = 1;
 
     mod->pat = hio_read8(f);
 
-    if ((mod->xxp = calloc(sizeof (struct xmp_pattern *), mod->pat)) == NULL)
+    mod->xxp = (struct xmp_pattern **) calloc(mod->pat, sizeof(struct xmp_pattern *));
+    if (mod->xxp == NULL)
         return -1;
 
     D_(D_INFO "Stored patterns: %d", mod->pat);
@@ -501,6 +506,7 @@ static int get_chunk_tr(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->has_tr || !data->has_pa) {
+	D_(D_CRIT "duplicate TR chunk or missing PA chunk");
 	return -1;
     }
     data->has_tr = 1;
@@ -519,13 +525,14 @@ static int get_chunk_tr(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	return -1;
     }
 
-    if ((mod->xxt = calloc(sizeof (struct xmp_track *), mod->trk)) == NULL)
+    mod->xxt = (struct xmp_track **) calloc(mod->trk, sizeof(struct xmp_track *));
+    if (mod->xxt == NULL)
 	return -1;
 
     D_(D_INFO "Stored tracks: %d", mod->trk);
 
-    track = calloc(1, sizeof (struct xmp_track) +
-			sizeof (struct xmp_event) * 255);
+    track = (struct xmp_track *) calloc(1, sizeof(struct xmp_track) +
+					   sizeof(struct xmp_event) * 255);
     if (track == NULL)
 	goto err;
 
@@ -537,8 +544,8 @@ static int get_chunk_tr(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	/* Length of the track in bytes */
 	len = hio_read16l(f);
 
-	memset(track, 0, sizeof (struct xmp_track) +
-            			sizeof (struct xmp_event) * 255);
+	memset(track, 0, sizeof(struct xmp_track) +
+			 sizeof(struct xmp_event) * 255);
 
 	for (row = 0; len;) {
 	    struct xmp_event *ev;
@@ -636,10 +643,11 @@ static int get_chunk_ii(struct module_data *m, int size, HIO_HANDLE *f, void *pa
     struct local_data *data = (struct local_data *)parm;
     int i, j, k;
     int map, last_map;
-    char buf[40];
+    uint8 buf[40];
 
     /* Sanity check */
     if (data->has_ii) {
+	D_(D_CRIT "duplicate II chunk");
 	return -1;
     }
     data->has_ii = 1;
@@ -647,7 +655,8 @@ static int get_chunk_ii(struct module_data *m, int size, HIO_HANDLE *f, void *pa
     mod->ins = hio_read8(f);
     D_(D_INFO "Instruments: %d", mod->ins);
 
-    if ((mod->xxi = calloc(sizeof (struct xmp_instrument), mod->ins)) == NULL)
+    mod->xxi = (struct xmp_instrument *) calloc(mod->ins, sizeof(struct xmp_instrument));
+    if (mod->xxi == NULL)
 	return -1;
 
     for (i = 0; i < mod->ins; i++) {
@@ -660,8 +669,7 @@ static int get_chunk_ii(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	    return -1;
 	}
 	buf[32] = 0;
-	strncpy(xxi->name, buf, 31);
-	xxi->name[31] = '\0';
+	libxmp_instrument_name(mod, i, buf, 32);
 
 	D_(D_INFO "[%2X] %-32.32s %2d", data->i_index[i], xxi->name, xxi->nsm);
 
@@ -669,7 +677,7 @@ static int get_chunk_ii(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	    return -1;
 
 	for (j = 0; j < XMP_MAX_KEYS; j++)
-	    xxi->map[j].ins = -1;
+	    xxi->map[j].ins = 0xff;
 
 	for (last_map = j = 0; j < mod->xxi[i].nsm; j++) {
 	    int x;
@@ -726,22 +734,25 @@ static int get_chunk_is(struct module_data *m, int size, HIO_HANDLE *f, void *pa
     struct xmp_module *mod = &m->mod;
     struct local_data *data = (struct local_data *)parm;
     int i;
-    char buf[64];
+    uint8 buf[64];
     uint8 x;
 
     /* Sanity check */
     if (data->has_is) {
+	D_(D_CRIT "duplicate IS chunk");
 	return -1;
     }
     data->has_is = 1;
 
     mod->smp = hio_read8(f);
-    if ((mod->xxs = calloc(sizeof (struct xmp_sample), mod->smp)) == NULL)
+    mod->xxs = (struct xmp_sample *) calloc(mod->smp, sizeof(struct xmp_sample));
+    if (mod->xxs == NULL)
 	return -1;
-    if ((m->xtra = calloc(sizeof (struct extra_sample_data), mod->smp)) == NULL)
+    m->xtra = (struct extra_sample_data *) calloc(mod->smp, sizeof(struct extra_sample_data));
+    if (m->xtra == NULL)
         return -1;
 
-    data->packinfo = calloc(sizeof (int), mod->smp);
+    data->packinfo = (int *) calloc(mod->smp, sizeof(int));
     if (data->packinfo == NULL)
 	return -1;
 
@@ -757,8 +768,7 @@ static int get_chunk_is(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	    return -1;
 	}
 	buf[32] = 0;
-	strncpy(xxs->name, buf, 31);
-	xxs->name[31] = '\0';
+	libxmp_copy_adjust(xxs->name, buf, 31);
 
 	hio_seek(f, 8, SEEK_CUR);		/* Sample filename */
 
@@ -768,6 +778,14 @@ static int get_chunk_is(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	xxs->lps = hio_read32l(f);
 	xxs->lpe = hio_read32l(f);
 
+	/* Sanity check */
+	if (xxs->len < 0 || xxs->lps < 0 ||
+	    xxs->lps > xxs->len || xxs->lpe > (xxs->len - xxs->lps)) {
+		D_(D_CRIT "invalid sample %d - len:%d s:%d l:%d",
+			i, xxs->len, xxs->lps, xxs->lpe);
+		return -1;
+	}
+
 	xxs->flg = xxs->lpe > 0 ? XMP_SAMPLE_LOOP : 0;
 	xxs->lpe = xxs->lps + xxs->lpe;
 
@@ -785,7 +803,7 @@ static int get_chunk_is(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	data->packinfo[i] = (x & 0x0c) >> 2;
 
 	D_(D_INFO "[%2X] %-32.32s %05x%c %05x %05x %c %6d %d",
-			data->s_index[i], buf, xxs->len,
+			data->s_index[i], xxs->name, xxs->len,
 			xxs->flg & XMP_SAMPLE_16BIT ? '+' : ' ',
 			xxs->lps, xxs->lpe,
 			xxs->flg & XMP_SAMPLE_LOOP ? 'L' : ' ',
@@ -800,11 +818,12 @@ static int get_chunk_i0(struct module_data *m, int size, HIO_HANDLE *f, void *pa
     struct xmp_module *mod = &m->mod;
     struct local_data *data = (struct local_data *)parm;
     int i;
-    char buf[64];
+    uint8 buf[64];
     uint8 x;
 
     /* Sanity check */
     if (data->has_ii || data->has_is) {
+	D_(D_CRIT "duplicate IS (0.0) chunk");
 	return -1;
     }
     data->has_ii = 1;
@@ -812,12 +831,13 @@ static int get_chunk_i0(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     mod->ins = mod->smp = hio_read8(f);
 
-    D_(D_INFO "Instruments: %d", mod->ins);
+    D_(D_INFO "Instruments (0.0): %d", mod->ins);
 
     if (libxmp_init_instrument(m) < 0)
 	return -1;
 
-    if ((data->packinfo = calloc(sizeof (int), mod->smp)) == NULL)
+    data->packinfo = (int *) calloc(mod->smp, sizeof(int));
+    if (data->packinfo == NULL)
 	return -1;
 
     for (i = 0; i < mod->ins; i++) {
@@ -839,14 +859,22 @@ static int get_chunk_i0(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	}
 	buf[32] = 0;
 	hio_seek(f, 8, SEEK_CUR);	/* Sample filename */
-	strncpy(xxi->name, buf, 31);
-	xxi->name[31] = '\0';
+	libxmp_instrument_name(mod, i, buf, 32);
 
 	c5spd = hio_read16l(f);
 
 	xxs->len = hio_read32l(f);
 	xxs->lps = hio_read32l(f);
 	xxs->lpe = hio_read32l(f);
+
+	/* Sanity check */
+	if (xxs->len < 0 || xxs->lps < 0 ||
+	    xxs->lps > xxs->len || xxs->lpe > (xxs->len - xxs->lps)) {
+		D_(D_CRIT "invalid sample %d - len:%d s:%d l:%d",
+			i, xxs->len, xxs->lps, xxs->lpe);
+		return -1;
+	}
+
 	xxs->flg = xxs->lpe > 0 ? XMP_SAMPLE_LOOP : 0;
 	xxs->lpe = xxs->lps + xxs->lpe;
 
@@ -866,7 +894,7 @@ static int get_chunk_i0(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	data->packinfo[i] = (x & 0x0c) >> 2;
 
 	D_(D_INFO "[%2X] %-32.32s %5d V%02x %05x%c %05x %05x %d",
-		data->i_index[i], buf, c5spd, sub->vol,
+		data->i_index[i], xxi->name, c5spd, sub->vol,
 		xxs->len, xxs->flg & XMP_SAMPLE_16BIT ? '+' : ' ',
 		xxs->lps, xxs->lpe, data->packinfo[i]);
     }
@@ -885,6 +913,7 @@ static int get_chunk_sa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->has_sa || !data->has_is || data->packinfo == NULL) {
+	D_(D_CRIT "duplicate SA chunk or missing IS chunk");
 	return -1;
     }
     data->has_sa = 1;
@@ -898,8 +927,6 @@ static int get_chunk_sa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	struct xmp_sample *xxs = &mod->xxs[i];
 
         len = xxs->len;
-	if (len < 0)
-	    goto err2;
 	if (xxs->flg & XMP_SAMPLE_16BIT)
 	    len <<= 1;
 
@@ -918,6 +945,7 @@ static int get_chunk_sa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	    break;
 	default:
 	    /* Sanity check */
+	    D_(D_CRIT "sample %d invalid pack %d", i, data->packinfo[i]);
 	    goto err2;
 	}
 
@@ -929,7 +957,7 @@ static int get_chunk_sa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 	}
 
 	if (len > smpbuf_alloc) {
-	    uint8 *tmp = realloc(smpbuf, len);
+	    uint8 *tmp = (uint8 *) realloc(smpbuf, len);
 	    if (!tmp)
 		goto err2;
 
@@ -939,8 +967,10 @@ static int get_chunk_sa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
 	switch (data->packinfo[i]) {
 	case 0:
-	    if (hio_read(smpbuf, 1, len, f) < len)
+	    if (hio_read(smpbuf, 1, len, f) < len) {
+		D_(D_CRIT "sample %d read error (no pack)", i);
 		goto err2;
+	    }
 	    left -= len;
 	    break;
 	case 1:
@@ -950,14 +980,18 @@ static int get_chunk_sa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
                 goto err2;
             if (len <= 0 || len > 0x80000)  /* Max compressed sample size */
                 goto err2;
-	    if ((buf = malloc(len + 4)) == NULL)
+	    if ((buf = (uint8 *)malloc(len + 4)) == NULL)
 		goto err2;
-	    if (hio_read(buf, 1, len, f) != len)
-                goto err3;
+	    if (hio_read(buf, 1, len, f) != len) {
+		D_(D_CRIT "sample %d read error (8-bit)", i);
+		goto err3;
+	    }
 	    /* The unpack function may read slightly beyond the end. */
 	    buf[len] = buf[len + 1] = buf[len + 2] = buf[len + 3] = 0;
-            if (unpack_sample8(smpbuf, buf, len, xxs->len) < 0)
-                goto err3;
+            if (unpack_sample8(smpbuf, buf, len, xxs->len) < 0) {
+		D_(D_CRIT "sample %d unpack error (8-bit)", i);
+		goto err3;
+	    }
 	    free(buf);
 	    left -= len + 4;
 	    break;
@@ -968,14 +1002,18 @@ static int get_chunk_sa(struct module_data *m, int size, HIO_HANDLE *f, void *pa
                 goto err2;
             if (len <= 0 || len > MAX_SAMPLE_SIZE)
                 goto err2;
-	    if ((buf = malloc(len + 4)) == NULL)
+	    if ((buf = (uint8 *)malloc(len + 4)) == NULL)
 		goto err2;
-	    if (hio_read(buf, 1, len, f) != len)
-                goto err3;
+	    if (hio_read(buf, 1, len, f) != len) {
+		D_(D_CRIT "sample %d read error (16-bit)", i);
+		goto err3;
+	    }
 	    /* The unpack function may read slightly beyond the end. */
 	    buf[len] = buf[len + 1] = buf[len + 2] = buf[len + 3] = 0;
-            if (unpack_sample16(smpbuf, buf, len, xxs->len) < 0)
-                goto err3;
+            if (unpack_sample16(smpbuf, buf, len, xxs->len) < 0) {
+		D_(D_CRIT "sample %d unpack error (16-bit)", i);
+		goto err3;
+	    }
 	    free(buf);
 	    left -= len + 4;
 	    break;
@@ -1001,6 +1039,7 @@ static int get_chunk_ve(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->v_env) {
+	D_(D_CRIT "duplicate VE chunk");
 	return -1;
     }
 
@@ -1009,7 +1048,7 @@ static int get_chunk_ve(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     D_(D_INFO "Vol envelopes: %d", data->v_envnum);
 
-    data->v_env = calloc(data->v_envnum, sizeof (struct mdl_envelope));
+    data->v_env = (struct mdl_envelope *) calloc(data->v_envnum, sizeof(struct mdl_envelope));
     if (data->v_env == NULL) {
 	return -1;
     }
@@ -1031,6 +1070,7 @@ static int get_chunk_pe(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->p_env) {
+	D_(D_CRIT "duplicate PE chunk");
 	return -1;
     }
 
@@ -1039,7 +1079,7 @@ static int get_chunk_pe(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     D_(D_INFO "Pan envelopes: %d", data->p_envnum);
 
-    data->p_env = calloc(data->p_envnum, sizeof (struct mdl_envelope));
+    data->p_env = (struct mdl_envelope *) calloc(data->p_envnum, sizeof(struct mdl_envelope));
     if (data->p_env == NULL) {
 	return -1;
     }
@@ -1061,6 +1101,7 @@ static int get_chunk_fe(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     /* Sanity check */
     if (data->f_env) {
+	D_(D_CRIT "duplicate FE chunk");
 	return -1;
     }
 
@@ -1069,7 +1110,7 @@ static int get_chunk_fe(struct module_data *m, int size, HIO_HANDLE *f, void *pa
 
     D_(D_INFO "Pitch envelopes: %d", data->f_envnum);
 
-    data->f_env = calloc(data->f_envnum, sizeof (struct mdl_envelope));
+    data->f_env = (struct mdl_envelope *) calloc(data->f_envnum, sizeof(struct mdl_envelope));
     if (data->f_env == NULL) {
 	return -1;
     }
@@ -1100,7 +1141,8 @@ static int mdl_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     /* Check magic and get version */
     hio_read32b(f);
-    hio_read(buf, 1, 1, f);
+    if (hio_read(buf, 1, 1, f) < 1)
+	return -1;
 
     handle = libxmp_iff_new();
     if (handle == NULL)
@@ -1136,11 +1178,11 @@ static int mdl_load(struct module_data *m, HIO_HANDLE *f, const int start)
     m->c4rate = C4_NTSC_RATE;
 
     data.v_envnum = data.p_envnum = data.f_envnum = 0;
-    data.s_index = calloc(256, sizeof (int));
-    data.i_index = calloc(256, sizeof (int));
-    data.v_index = malloc(256 * sizeof (int));
-    data.p_index = malloc(256 * sizeof (int));
-    data.f_index = malloc(256 * sizeof (int));
+    data.s_index = (int *) calloc(256, sizeof(int));
+    data.i_index = (int *) calloc(256, sizeof(int));
+    data.v_index = (int *) malloc(256 * sizeof(int));
+    data.p_index = (int *) malloc(256 * sizeof(int));
+    data.f_index = (int *) malloc(256 * sizeof(int));
     if (!data.s_index || !data.i_index || !data.v_index || !data.p_index || !data.f_index) {
 	goto err;
     }
@@ -1151,7 +1193,7 @@ static int mdl_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     /* Load IFFoid chunks */
     if (libxmp_iff_load(handle, m, f, &data) < 0) {
-    	libxmp_iff_release(handle);
+	libxmp_iff_release(handle);
 	retval = -1;
 	goto err;
     }
