@@ -27,12 +27,6 @@
  */
 
 #include "loader.h"
-#include <sys/stat.h>
-#ifdef __native_client__
-#include <sys/syslimits.h>
-#else
-#include <limits.h>
-#endif
 
 static int mfp_test(HIO_HANDLE *, char *, const int);
 static int mfp_load(struct module_data *, HIO_HANDLE *, const int);
@@ -102,8 +96,7 @@ static int mfp_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	struct xmp_module *mod = &m->mod;
 	int i, j, k, x, y;
 	struct xmp_event *event;
-	struct stat st;
-	char smp_filename[PATH_MAX];
+	char smp_filename[XMP_MAXPATH];
 	HIO_HANDLE *s;
 	int size1 /*, size2*/;
 	int pat_addr, pat_table[128][4];
@@ -142,11 +135,11 @@ static int mfp_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		if (mod->xxs[i].len > 0)
 			mod->xxi[i].nsm = 1;
 
-               	D_(D_INFO "[%2X] %04x %04x %04x %c V%02x %+d",
-                       	i, mod->xxs[i].len, mod->xxs[i].lps,
-                       	mod->xxs[i].lpe,
+		D_(D_INFO "[%2X] %04x %04x %04x %c V%02x %+d",
+			i, mod->xxs[i].len, mod->xxs[i].lps,
+			mod->xxs[i].lpe,
 			loop_size > 1 ? 'L' : ' ',
-                       	mod->xxi[i].sub[0].vol, mod->xxi[i].sub[0].fin >> 4);
+			mod->xxi[i].sub[0].vol, mod->xxi[i].sub[0].fin >> 4);
 	}
 
 	mod->len = mod->pat = hio_read8(f);
@@ -215,33 +208,26 @@ static int mfp_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	/* first check smp.filename */
 	if (strlen(m->basename) < 5 || m->basename[3] != '.') {
-		fprintf(stderr, "libxmp: invalid filename %s\n", m->basename);
+		D_(D_CRIT "invalid filename %s", m->basename);
 		goto err;
 	}
 
 	m->basename[0] = 's';
 	m->basename[1] = 'm';
 	m->basename[2] = 'p';
-	snprintf(smp_filename, PATH_MAX, "%s%s", m->dirname, m->basename);
-	if (stat(smp_filename, &st) < 0) {
+	snprintf(smp_filename, XMP_MAXPATH, "%s%s", m->dirname, m->basename);
+	if ((s = hio_open(smp_filename, "rb")) == NULL) {
 		/* handle .set filenames like in Kid Chaos*/
-		char *x;
 		if (strchr(m->basename, '-')) {
-			x = strrchr(smp_filename, '-');
-			if (x != NULL)
-				strcpy(x, ".set");
+			char *p = strrchr(smp_filename, '-');
+			if (p != NULL)
+				strcpy(p, ".set");
 		}
-		if (stat(smp_filename, &st) < 0) {
-			fprintf(stderr, "libxmp: missing file %s\n",
-								smp_filename);
+		if ((s = hio_open(smp_filename, "rb")) == NULL) {
+			D_(D_CRIT "can't open sample file %s", smp_filename);
 			goto err;
 		}
 	}
-	if ((s = hio_open(smp_filename, "rb")) == NULL) {
-		fprintf(stderr, "libxmp: can't open sample file %s\n",
-								smp_filename);
-		goto err;
-	}
 
 	for (i = 0; i < mod->ins; i++) {
 		if (libxmp_load_sample(m, s, SAMPLE_FLAG_FULLREP,
