@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2023 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2025 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -175,6 +175,7 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	 * convert header
 	 */
 	m->c4rate = C4_NTSC_RATE;
+	m->quirk |= QUIRK_RTONCE; /* FF1 */
 	m->quirk |= song.flags & FLAG_STSLIDE ? 0 : QUIRK_VSALL | QUIRK_PBALL;
 	hexvol = song.flags & FLAG_VOLHEX;
 	med_8ch = song.flags & FLAG_8CHANNEL;
@@ -379,9 +380,7 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	mod->trk = mod->pat * mod->chn;
 
-	libxmp_set_type(m, ver == 0 ? mmdc ? "MED Packer MMDC" :
-				mod->chn > 4 ? "OctaMED 2.00 MMD0" :
-				"MED 2.10 MMD0" : "OctaMED 4.00 MMD1");
+	mmd_tracker_version(m, ver, mmdc, expdata_offset ? &expdata : NULL);
 
 	MODULE_INFO();
 
