@@ -1,9 +1,31 @@
+/* ProWizard
+ * Copyright (C) 1997 Asle / ReDoX
+ * Modified in 2006,2007,2014 by Claudio Matsuoka
+ * Modified in 2021 by Alice Rowan
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
+ */
+
 /*
- * Module_Protector.c   Copyright (C) 1997 Asle / ReDoX
+ * Module_Protector.c
  *
  * Converts MP packed MODs back to PTK MODs
- *
- * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
 #include "prowiz.h"
@@ -124,15 +146,15 @@ static int test_mp_noid(const uint8 *data, char *t, int s)
 	/* test #5  ptk notes .. gosh ! (testing all patterns !) */
 	for (i = 0; i < psize; i++) {
 		const uint8 *d = data + 378 + i * 4;
-		uint16 data;
+		uint16 val;
 
 		/* MadeInCroatia has l == 74 */
 		if (*d > 19 && *d != 74)
 			return -1;
 
-		data = readmem16b(d) & 0x0fff;
+		val = readmem16b(d) & 0x0fff;
 
-		if (data > 0 && data < 0x71)
+		if (val > 0 && val < 0x71)
 			return -1;
 	}
 
