@@ -1,11 +1,33 @@
+/* ProWizard
+ * Copyright (C) 1997 Asle / ReDoX
+ * Modified in 2006,2007,2014 by Claudio Matsuoka
+ * Modified in 2021 by Alice Rowan
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
+ */
+
 /*
- * Promizer_18a.c   Copyright (C) 1997 Asle / ReDoX
+ * Promizer_18a.c
  *
  * Converts PM18a packed MODs back to PTK MODs
  * thanks to Gryzor and his ProWizard tool ! ... without it, this prog
  * would not exist !!!
- *
- * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
 #include "prowiz.h"
@@ -39,7 +61,13 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 	pw_write_zero(out, 20);				/* title */
 
 	/* bypass replaycode routine */
-	hio_seek(in, 4464, SEEK_SET);
+	hio_seek(in, 4460, SEEK_SET);
+	psize = hio_read32b(in);
+
+	/* Sanity check */
+	if (psize < 0) {
+		return -1;
+	}
 
 	ssize = 0;
 	for (i = 0; i < 31; i++) {
@@ -62,9 +90,16 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 	write8(out, num_pat);
 	write8(out, 0x7f);				/* NoiseTracker byte */
 
-	for (i = 0; i < 128; i++)
+	for (i = 0; i < 128; i++) {
 		paddr[i] = hio_read32b(in);
 
+		/* Sanity check */
+		if (paddr[i] < 0 || paddr[i] - 5226 > psize) {
+			return -1;
+		}
+	}
+	/* At 5226 now, the start of the pattern data. */
+
 	/* ordering of patterns addresses */
 
 	pat_max = 0;
@@ -90,9 +125,6 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 	/* a little pre-calc code ... no other way to deal with these unknown
 	 * pattern data sizes ! :(
 	 */
-	hio_seek(in, 4460, SEEK_SET);
-	psize = hio_read32b(in);
-	hio_seek(in, 5226, SEEK_SET);	/* back to pattern data start */
 
 	/* now, reading all pattern data to get the max value of note */
 	refmax = 0;
@@ -120,7 +152,7 @@ static int depack_p18a(HIO_HANDLE *in, FILE *out)
 
 	for (j = 0; j <= pat_max; j++) {
 		int flag = 0;
-		hio_seek(in, paddr[j] + 5226, 0);
+		hio_seek(in, paddr[j] + 5226, SEEK_SET);
 		for (i = 0; i < 64; i++) {
 			for (k = 0; k < 4; k++) {
 				uint8 *p = &pat[j][i * 16 + k * 4];
