@@ -1,10 +1,10 @@
 #ifndef PROWIZ_H
 #define PROWIZ_H
 
-#include "list.h"
-#include "common.h"
-#include "format.h"
-#include "hio.h"
+#include "../../common.h"
+#include "../../format.h"
+#include "../../hio.h"
+/*#include "../../list.h"*/
 
 #define MIN_FILE_LENGHT 2048
 
@@ -30,7 +30,7 @@ struct pw_format {
 	const char *name;
 	int (*test)(const uint8 *, char *, int);
 	int (*depack)(HIO_HANDLE *, FILE *);
-	struct list_head list;
+	/*struct list_head list;*/
 };
 
 int pw_wizardry(HIO_HANDLE *, FILE *, const char **);
