@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2022 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -97,7 +97,7 @@ static int get_info(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	mod->len = hio_read16b(f);
 	mod->pat = hio_read16b(f);
 	mod->gvl = hio_read16b(f);
-	mod->bpm = hio_read16b(f);
+	mod->bpm = hio_read16b(f);	/* Not clamped by Protracker 3.6 */
 	/*flags =*/ hio_read16b(f);
 	/*day   =*/ hio_read16b(f);
 	/*month =*/ hio_read16b(f);
@@ -162,7 +162,8 @@ static int pt3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_read32b(f);		/* VERS */
 	hio_read32b(f);		/* VERS size */
 
-	hio_read(buf, 1, 10, f);
+	if (hio_read(buf, 1, 10, f) < 10)
+		return -1;
 	libxmp_set_type(m, "%-6.6s IFFMODL", buf + 4);
 
 	handle = libxmp_iff_new();
