@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2025 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -313,6 +313,9 @@ static int ptm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 					event->fxt = event->fxp = 0;
 
 				switch (event->fxt) {
+				case 0x0d:	/* Break (hex parameter) */
+					event->fxp = FX_IT_BREAK;
+					break;
 				case 0x0e:	/* Extended effect */
 					if (MSN(event->fxp) == 0x8) {	/* Pan set */
 						event->fxt = FX_SETPAN;
@@ -372,7 +375,11 @@ static int ptm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		mod->xxc[i].pan = pfh.chset[i] << 4;
 
 	m->quirk |= QUIRKS_ST3;
-	m->flow_mode = FLOW_LOOP_GLOBAL; /* Has none of ST3's loop quirks. */
+	/* Has none of ST3's loop quirks; loop jumps unset prior breaks.
+	 * TODO: there is an obscure bug where loop jumps take precedence over
+	 * position jumps *ONLY WHEN THE PLAYER IS AT SPEED 1*.
+	 * TODO: jumps are always to row 0. */
+	m->flow_mode = FLOW_LOOP_GLOBAL | FLOW_LOOP_UNSET_BREAK;
 	m->read_event_type = READ_EVENT_ST3;
 
 	return 0;
