@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 2023-2024 Alice Rowan <petrifiedrowan@gmail.com>
+ * Copyright (C) 2023-2025 Alice Rowan <petrifiedrowan@gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -211,7 +211,6 @@ static void xmf_translate_effect(struct xmp_event *event, uint8 effect, uint8 pa
 		switch (param >> 4) {
 		case 0x01:		/* fine slide up */
 		case 0x02:		/* fine slide down */
-		case 0x06:		/* pattern loop (broken) */
 		case 0x09:		/* note retrigger (TODO: only once) */
 		case 0x0c:		/* note cut */
 		case 0x0d:		/* note delay */
@@ -227,6 +226,10 @@ static void xmf_translate_effect(struct xmp_event *event, uint8 effect, uint8 pa
 			xmf_insert_effect(event, effect, param, chn);
 			break;
 
+		case 0x06:		/* pattern loop */
+			xmf_insert_effect(event, effect, param, chn);
+			break;
+
 		case 0x0a:		/* fine volume slide up */
 			if (param & 0x0f) {
 				xmf_insert_effect(event, FX_F_VSLIDE_UP, (param & 0x0f) << 2, chn);
