@@ -2,18 +2,19 @@
 #define LIBXMP_MEMIO_H
 
 #include <stddef.h>
+#include "common.h"
 
 typedef struct {
 	const unsigned char *start;
 	ptrdiff_t pos;
 	ptrdiff_t size;
+	void *ptr_free;
 } MFILE;
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+LIBXMP_BEGIN_DECLS
 
-MFILE  *mopen(const void *, long);
+MFILE  *mopen(void *, long, int);
+MFILE  *mcopen(const void *, long);
 int     mgetc(MFILE *stream);
 size_t  mread(void *, size_t, size_t, MFILE *);
 int     mseek(MFILE *, long, int);
@@ -21,8 +22,6 @@ long    mtell(MFILE *);
 int     mclose(MFILE *);
 int	meof(MFILE *);
 
-#ifdef __cplusplus
-}
-#endif
+LIBXMP_END_DECLS
 
 #endif
