@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +32,7 @@
 /*
  * Period table from the Protracker V2.1A play routine
  */
-static uint16 pt_period_table[16][36] = {
+static const uint16 pt_period_table[16][36] = {
 	/* Tuning 0, Normal */
 	{
 		856,808,762,720,678,640,604,570,538,508,480,453,
