@@ -20,5 +20,8 @@ double	libxmp_note_to_period_mix (int, int);
 int	libxmp_period_to_note	(int);
 int	libxmp_period_to_bend	(struct context_data *, double, int, double);
 void	libxmp_c2spd_to_note	(int, int *, int *);
+#ifndef LIBXMP_CORE_PLAYER
+double	libxmp_gus_frequency_steps (int, int);
+#endif
 
 #endif /* LIBXMP_PERIOD_H */
