@@ -1,4 +1,4 @@
-static int16 cubic_spline_lut0[1024] = {
+static const int16 cubic_spline_lut0[1024] = {
 	 0, -8, -16, -24, -32, -40, -47, -55,
 	 -63, -71, -78, -86, -94, -101, -109, -117,
 	 -124, -132, -139, -146, -154, -161, -169, -176,
@@ -129,7 +129,7 @@ static int16 cubic_spline_lut0[1024] = {
 	 0, 0, 0, 0, 0, 0, 0, 0,
 };
 
-static int16 cubic_spline_lut1[1024] = {
+static const int16 cubic_spline_lut1[1024] = {
 	 16384, 16384, 16384, 16384, 16384, 16383, 16382, 16381,
 	 16381, 16381, 16380, 16379, 16379, 16377, 16377, 16376,
 	 16374, 16373, 16371, 16370, 16369, 16366, 16366, 16364,
@@ -260,7 +260,7 @@ static int16 cubic_spline_lut1[1024] = {
 	 66, 58, 49, 41, 32, 24, 16, 8,
 };
 
-static int16 cubic_spline_lut2[1024] = {
+static const int16 cubic_spline_lut2[1024] = {
 	 0, 8, 16, 24, 32, 41, 49, 58,
 	 66, 75, 83, 92, 100, 109, 118, 127,
 	 136, 145, 154, 163, 172, 182, 191, 200,
@@ -391,7 +391,7 @@ static int16 cubic_spline_lut2[1024] = {
 	 16381, 16381, 16382, 16383, 16384, 16384, 16384, 16384,
 };
 
-static int16 cubic_spline_lut3[1024] = {
+static const int16 cubic_spline_lut3[1024] = {
 	 0, 0, 0, 0, 0, 0, 0, 0,
 	 0, -1, -1, -1, -1, -1, -2, -2,
 	 -2, -2, -2, -3, -3, -3, -4, -4,
