@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,17 +25,13 @@
 #include "effects.h"
 #include "virtual.h"
 #include "period.h"
+#include "rng.h"
 
 #ifndef LIBXMP_CORE_PLAYER
 #include "med_extras.h"
 #endif
 
 
-static inline int is_valid_note(int note)
-{
-	return (note >= 0 && note < XMP_MAX_KEYS);
-}
-
 static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 						   int ins, int key)
 {
@@ -45,7 +41,7 @@ static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 
 	if (IS_VALID_INSTRUMENT(ins)) {
 		instrument = &mod->xxi[ins];
-		if (is_valid_note(key)) {
+		if (IS_VALID_NOTE(key)) {
 			int mapped = instrument->map[key].ins;
 			if (mapped != 0xff && mapped >= 0 && mapped < instrument->nsm)
 			  	return &instrument->sub[mapped];
@@ -67,7 +63,7 @@ static void reset_envelopes(struct context_data *ctx, struct channel_data *xc)
 	if (!IS_VALID_INSTRUMENT(xc->ins))
 		return;
 
- 	RESET_NOTE(NOTE_ENV_END);
+	RESET_NOTE(NOTE_ENV_END);
 
 	xc->v_idx = -1;
 	xc->p_idx = -1;
@@ -76,6 +72,20 @@ static void reset_envelopes(struct context_data *ctx, struct channel_data *xc)
 
 #ifndef LIBXMP_CORE_DISABLE_IT
 
+static void reset_envelope_volume(struct context_data *ctx,
+				struct channel_data *xc)
+{
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
+
+	if (!IS_VALID_INSTRUMENT(xc->ins))
+		return;
+
+	RESET_NOTE(NOTE_ENV_END);
+
+	xc->v_idx = -1;
+}
+
 static void reset_envelopes_carry(struct context_data *ctx,
 				struct channel_data *xc)
 {
@@ -109,18 +119,8 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 				struct channel_data *xc, int is_toneporta)
 {
 	struct module_data *m = &ctx->m;
-	struct xmp_module *mod = &m->mod;
-	struct smix_data *smix = &ctx->smix;
-	
+
 	if (sub != NULL && note >= 0) {
-		struct xmp_instrument *xxi;
-
-		if (xc->ins >= mod->ins) {
-			xxi = &smix->xxi[xc->ins - mod->ins];
-		} else {
-			xxi = &mod->xxi[xc->ins];
-		}
-
 		if (!HAS_QUIRK(QUIRK_PROTRACK)) {
 			xc->finetune = sub->fin;
 		}
@@ -129,16 +129,16 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 #ifndef LIBXMP_CORE_DISABLE_IT
 		if (sub->ifc & 0x80) {
 			xc->filter.cutoff = (sub->ifc - 0x80) * 2;
-		} else if (~xxi->fei.flg & XMP_ENVELOPE_FLT) {
-			xc->filter.cutoff = 0xff;
 		}
 		xc->filter.envelope = 0x100;
 
 		if (sub->ifr & 0x80) {
 			xc->filter.resonance = (sub->ifr - 0x80) * 2;
-		} /* else {
-			xc->filter.resonance = 0;
-		} */
+		}
+
+		/* IT: on a new note without toneporta, allow a computed cutoff
+		 * of 127 with resonance 0 to disable the filter. */
+		xc->filter.can_disable = !is_toneporta;
 #endif
 
 		/* TODO: should probably expand the LFO period size instead
@@ -176,7 +176,13 @@ static void set_period(struct context_data *ctx, int note,
 {
 	struct module_data *m = &ctx->m;
 
-	if (sub != NULL && note >= 0) {
+	/* TODO: blocking period updates on whether or not the event has a
+	 * valid instrument seems suspicious, but almost every format uses
+	 * this. Only allow Protracker to update without it for now. */
+	if (sub == NULL && !HAS_QUIRK(QUIRK_PROTRACK))
+		return;
+
+	if (note >= 0) {
 		double per = libxmp_note_to_period(ctx, note, xc->finetune,
 							xc->per_adj);
 
@@ -215,13 +221,16 @@ static void set_period_ft2(struct context_data *ctx, int note,
 #ifndef LIBXMP_CORE_PLAYER
 #define IS_SFX_PITCH(x) ((x) == FX_PITCH_ADD || (x) == FX_PITCH_SUB)
 #define IS_TONEPORTA(x) ((x) == FX_TONEPORTA || (x) == FX_TONE_VSLIDE \
-		|| (x) == FX_PER_TPORTA)
+		|| (x) == FX_PER_TPORTA || (x) == FX_ULT_TPORTA \
+		|| (x) == FX_FAR_TPORTA)
 #else
 #define IS_TONEPORTA(x) ((x) == FX_TONEPORTA || (x) == FX_TONE_VSLIDE)
 #endif
 
+#define IS_MOD_RETRIG(x,p) ((x) == FX_EXTENDED && MSN(p) == EX_RETRIG && LSN(p) != 0)
+
 #define set_patch(ctx,chn,ins,smp,note) \
-	libxmp_virt_setpatch(ctx, chn, ins, smp, note, 0, 0, 0)
+	libxmp_virt_setpatch(ctx, chn, ins, smp, note, 0, 0, 0, 0)
 
 static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn)
 {
@@ -230,19 +239,25 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 	struct xmp_module *mod = &m->mod;
 	struct channel_data *xc = &p->xc_data[chn];
 	int note;
-	struct xmp_subinstrument *sub;
+	struct xmp_subinstrument *sub = NULL;
 	int new_invalid_ins = 0;
+	int new_swap_ins = 0;
 	int is_toneporta;
+	int is_retrig;
 	int use_ins_vol;
 
 	xc->flags = 0;
 	note = -1;
 	is_toneporta = 0;
+	is_retrig = 0;
 	use_ins_vol = 0;
 
 	if (IS_TONEPORTA(e->fxt) || IS_TONEPORTA(e->f2t)) {
 		is_toneporta = 1;
 	}
+	if (IS_MOD_RETRIG(e->fxt, e->fxp) || IS_MOD_RETRIG(e->f2t, e->f2p)) {
+		is_retrig = 1;
+	}
 
 	/* Check instrument */
 
@@ -258,6 +273,18 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		if (IS_VALID_INSTRUMENT(ins)) {
 			sub = get_subinstrument(ctx, ins, e->note - 1);
 
+			if (sub != NULL) {
+				new_swap_ins = 1;
+
+				/* Finetune is always loaded, but only applies
+				 * when the period is updated by a note/porta
+				 * (OpenMPT finetune.mod, PortaSwapPT.mod). */
+				if (HAS_QUIRK(QUIRK_PROTRACK)) {
+					xc->finetune = sub->fin;
+					xc->ins = ins;
+				}
+			}
+
 			if (is_toneporta) {
 				/* Get new instrument volume */
 				if (sub != NULL) {
@@ -272,16 +299,26 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 			} else {
 				xc->ins = ins;
 				xc->ins_fade = mod->xxi[ins].rls;
-
-				if (sub != NULL) {
-					if (HAS_QUIRK(QUIRK_PROTRACK)) {
-						xc->finetune = sub->fin;
-					}
-				}
 			}
 		} else {
 			new_invalid_ins = 1;
-			libxmp_virt_resetchannel(ctx, chn);
+
+			/* Invalid instruments do not reset the channel in
+			 * Protracker; instead, they set the current sample
+			 * to the invalid sample, which stops the current
+			 * sample at the end of its loop.
+			 *
+			 * OpenMPT PTInstrSwap.mod: uses a null sample to pause
+			 * a looping sample, plays several on a channel with no note.
+			 *
+			 * OpenMPT PTSwapEmpty.mod: repeatedly pauses and
+			 * restarts a sample using a null sample.
+			 */
+			if (!HAS_QUIRK(QUIRK_PROTRACK) || is_retrig) {
+				libxmp_virt_resetchannel(ctx, chn);
+			} else {
+				libxmp_virt_queuepatch(ctx, chn, -1, -1, 0);
+			}
 		}
 	}
 
@@ -293,32 +330,52 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		if (e->note == XMP_KEY_OFF) {
 			SET_NOTE(NOTE_RELEASE);
 			use_ins_vol = 0;
-		} else if (!is_toneporta && is_valid_note(e->note - 1)) {
+		} else if (!is_toneporta && IS_VALID_NOTE(e->note - 1)) {
 			xc->key = e->note - 1;
 			RESET_NOTE(NOTE_END);
 
 			sub = get_subinstrument(ctx, xc->ins, xc->key);
 
-			if (!new_invalid_ins && sub != NULL) {
+			if (sub != NULL) {
 				int transp = mod->xxi[xc->ins].map[xc->key].xpo;
 				int smp;
 
 				note = xc->key + sub->xpo + transp;
 				smp = sub->sid;
 
-				if (mod->xxs[smp].len == 0) {
+				if (new_invalid_ins || !IS_VALID_SAMPLE(smp)) {
 					smp = -1;
 				}
 
 				if (smp >= 0 && smp < mod->smp) {
 					set_patch(ctx, chn, xc->ins, smp, note);
+					new_swap_ins = 0;
 					xc->smp = smp;
 				}
 			} else {
 				xc->flags = 0;
 				use_ins_vol = 0;
+				note = xc->key;
 			}
 		}
+
+		if (note >= 0) {
+			xc->note = note;
+			SET_NOTE(NOTE_SET);
+		}
+	}
+
+	/* Protracker 1/2 sample swap occurs when a sample number is
+	 * encountered without a note or with a note and toneporta. The new
+	 * instrument is switched to when the current sample reaches its loop
+	 * end. A valid note must have been played in this channel before.
+	 *
+	 * Empty samples can also be set, which stops the sample at the end
+	 * of its loop (see above).
+	 */
+	if (new_swap_ins && sub && HAS_QUIRK(QUIRK_PROTRACK) && TEST_NOTE(NOTE_SET)) {
+		libxmp_virt_queuepatch(ctx, chn, e->ins - 1, sub->sid, xc->note);
+		xc->smp = sub->sid;
 	}
 
 	sub = get_subinstrument(ctx, xc->ins, xc->key);
@@ -332,6 +389,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 	if (e->vol) {
 		xc->volume = e->vol - 1;
 		SET(NEW_VOL);
+		RESET_PER(VOL_SLIDE); /* FIXME: should this be for FAR only? */
 	}
 
 	/* Secondary effect handled first */
@@ -352,9 +410,13 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		return 0;
 	}
 
-	if (note >= 0) {
-		xc->note = note;
+	if (note >= 0 && !new_invalid_ins) {
 		libxmp_virt_voicepos(ctx, chn, xc->offset.val);
+	} else if (new_swap_ins && is_retrig && HAS_QUIRK(QUIRK_PROTRACK)) {
+		/* Protracker: an instrument number with no note and retrigger
+		 * triggers the new sample on tick 0. Other effects that set
+		 * RETRIG should not. (OpenMPT InstrSwapRetrigger.mod) */
+		libxmp_virt_voicepos(ctx, chn, 0);
 	}
 
 	if (TEST(OFFSET)) {
@@ -375,6 +437,7 @@ static int sustain_check(struct xmp_envelope *env, int idx)
 {
 	return (env &&
 		(env->flg & XMP_ENVELOPE_ON) &&
+		(env->flg & XMP_ENVELOPE_SUS) &&
 		(~env->flg & XMP_ENVELOPE_LOOP) &&
 		idx == env->data[env->sus << 1]);
 }
@@ -457,19 +520,17 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	/* FT2: Retrieve old instrument volume */
 	if (ins) {
 		if (key == 0 || key >= XMP_KEY_OFF) {
-			struct xmp_subinstrument *sub;
-
 			/* Previous instrument */
 			sub = get_subinstrument(ctx, xc->ins, xc->key);
 
 			/* No note */
 			if (sub != NULL) {
-				int p = mod->xxc[chn].pan - 128;
+				int pan = mod->xxc[chn].pan - 128;
 				xc->volume = sub->vol;
 
 				if (!HAS_QUIRK(QUIRK_FTMOD)) {
-					xc->pan.val = p + ((sub->pan - 128) *
-						(128 - abs(p))) / 128 + 128;
+					xc->pan.val = pan + ((sub->pan - 128) *
+						(128 - abs(pan))) / 128 + 128;
 				}
 
 				xc->ins_fade = mod->xxi[xc->ins].rls;
@@ -516,11 +577,8 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	/* Check note */
-
 	if (ins) {
 		if (key > 0 && key < XMP_KEY_OFF) {
-			struct xmp_subinstrument *sub;
-
 			/* Retrieve volume when we have note */
 
 			/* and only if we have instrument, otherwise we're in
@@ -530,12 +588,12 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			/* Current instrument */
 			sub = get_subinstrument(ctx, xc->ins, key - 1);
 			if (sub != NULL) {
-				int p = mod->xxc[chn].pan - 128;
+				int pan = mod->xxc[chn].pan - 128;
 				xc->volume = sub->vol;
 
 				if (!HAS_QUIRK(QUIRK_FTMOD)) {
-					xc->pan.val = p + ((sub->pan - 128) *
-						(128 - abs(p))) / 128 + 128;
+					xc->pan.val = pan + ((sub->pan - 128) *
+						(128 - abs(pan))) / 128 + 128;
 				}
 
 				xc->ins_fade = mod->xxi[xc->ins].rls;
@@ -570,7 +628,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 					env_on = 1;
 				}
 			}
-			
+
 			if (env_on || (!vol_set && (!ev.ins || !delay_fx))) {
 				if (sustain_check(env, xc->v_idx)) {
 					/* See OpenMPT EnvOff.xm. In certain
@@ -627,7 +685,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	 *  and remains in the memory."
 	 */
 	sub = NULL;
-	if (is_valid_note(key - 1)) {
+	if (IS_VALID_NOTE(key - 1)) {
 		int k = key - 1;
 		sub = get_subinstrument(ctx, xc->ins, k);
 		if (!new_invalid_ins && sub != NULL) {
@@ -640,7 +698,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		}
 	}
 
-	if (is_valid_note(key - 1)) {
+	if (IS_VALID_NOTE(key - 1)) {
 		xc->key = --key;
 		xc->fadeout = 0x10000;
 		RESET_NOTE(NOTE_END);
@@ -658,7 +716,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			note = key + sub->xpo + transp;
 			smp = sub->sid;
 
-			if (mod->xxs[smp].len == 0) {
+			if (!IS_VALID_SAMPLE(smp)) {
 				smp = -1;
 			}
 
@@ -814,7 +872,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 			if (not_same_ins) {
 				xc->offset.val = 0;
 			}
-		} else if (is_valid_note(e->note - 1)) {
+		} else if (IS_VALID_NOTE(e->note - 1)) {
 			xc->key = e->note - 1;
 			RESET_NOTE(NOTE_END);
 
@@ -827,7 +885,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 				note = xc->key + sub->xpo + transp;
 				smp = sub->sid;
 
-				if (mod->xxs[smp].len == 0) {
+				if (!IS_VALID_SAMPLE(smp)) {
 					smp = -1;
 				}
 
@@ -961,10 +1019,10 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	int is_toneporta, is_release;
 	int candidate_ins;
 	int reset_env;
+	int reset_susloop;
 	int use_ins_vol;
 	int sample_mode;
 	int toneporta_offset;
-	int disabled_toneporta;
 	int retrig_ins;
 	struct xmp_event ev;
 
@@ -987,11 +1045,11 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	is_toneporta = 0;
 	is_release = 0;
 	reset_env = 0;
+	reset_susloop = 0;
 	use_ins_vol = 0;
 	candidate_ins = xc->ins;
 	sample_mode = !HAS_QUIRK(QUIRK_VIRTUAL);
 	toneporta_offset = 0;
-	disabled_toneporta = 0;
 	retrig_ins = 0;
 
 	/* Keyoff + instrument retrigs current instrument in old fx mode */
@@ -1025,7 +1083,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		is_toneporta = 1;
 	}
 
-	if (TEST_NOTE(NOTE_RELEASE | NOTE_FADEOUT)) {
+	if (TEST_NOTE(NOTE_ENV_RELEASE | NOTE_FADEOUT)) {
 		is_release = 1;
 	}
 
@@ -1035,10 +1093,8 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	/* Off-Porta.it */
 	if (is_toneporta && ev.fxt == FX_OFFSET) {
-		disabled_toneporta = 1;
-		is_toneporta = 0;
+		toneporta_offset = 1;
  		if (!HAS_QUIRK(QUIRK_PRENV)) {
-			toneporta_offset = 1;
 			RESET_NOTE(NOTE_ENV_END);
 		}
 	}
@@ -1077,9 +1133,11 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 		if (set_new_ins) {
 			SET(NEW_INS);
-			use_ins_vol = 1;
 			reset_env = 1;
 		}
+		/* Sample default volume is always enabled if a valid sample
+		 * is provided (Atomic Playboy, default_volume.it). */
+		use_ins_vol = 1;
 		xc->per_flags = 0;
 
 		if (IS_VALID_INSTRUMENT(ins)) {
@@ -1119,7 +1177,10 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 				}
 			}
 		} else {
-			/* In sample mode invalid ins cut previous ins */
+			/* In sample mode invalid instruments cut the current
+			 * note (OpenMPT SampleNumberChange.it).
+			 * TODO: portamento_sustain.it order 3 row 19: when
+			 * sample release is set, this isn't always done? */
 			if (sample_mode) {
 				xc->volume = 0;
 			}
@@ -1133,13 +1194,14 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	/* Check note */
 
-	if (key && !new_invalid_ins) {
+	if (key) {
 		SET(NEW_NOTE);
 		SET_NOTE(NOTE_SET);
 
 		if (key == XMP_KEY_FADE) {
 			SET_NOTE(NOTE_FADEOUT);
 			reset_env = 0;
+			reset_susloop = 0;
 			use_ins_vol = 0;
 		} else if (key == XMP_KEY_CUT) {
 			SET_NOTE(NOTE_END | NOTE_CUT | NOTE_KEY_CUT);
@@ -1161,24 +1223,27 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			 * However, never reset the envelope (see OpenMPT wnoteoff.it).
 			 */
 			reset_env = 0;
+			reset_susloop = 0;
 			if (!ev.ins) {
 				use_ins_vol = 0;
 			}
-		} else {
+		} else if (!new_invalid_ins) {
+			/* Sample sustain release should always carry for tone
+			 * portamento, and is not reset unless a note is
+			 * present (Atomic Playboy, portamento_sustain.it). */
 			/* portamento_after_keyoff.it test case */
 			/* also see suburban_streets o13 c45 */
-			if (ev.ins || !is_toneporta) {
-				if (!disabled_toneporta) {
-					reset_env = 1;
-				}
+			if (!is_toneporta) {
+				reset_env = 1;
+				reset_susloop = 1;
 			}
 
 			if (is_toneporta) {
 				if (not_same_ins || TEST_NOTE(NOTE_END)) {
 					SET(NEW_INS);
-					RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
+					RESET_NOTE(NOTE_ENV_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
 				} else {
-					if (is_valid_note(key - 1)) {
+					if (IS_VALID_NOTE(key - 1)) {
 						xc->key_porta = key - 1;
 					}
 					key = 0;
@@ -1187,7 +1252,9 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		}
 	}
 
-	if (is_valid_note(key - 1) && !new_invalid_ins) {
+	/* TODO: instrument change+porta(+release?) doesn't require a key.
+	 * Order 3/row 11 of portamento_sustain.it should change the sample. */
+	if (IS_VALID_NOTE(key - 1) && !new_invalid_ins) {
 		if (TEST_NOTE(NOTE_CUT)) {
 			use_ins_vol = 1;	/* See OpenMPT NoteOffInstr.it */
 		}
@@ -1199,26 +1266,36 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		if (sub != NULL) {
 			int transp = mod->xxi[candidate_ins].map[key].xpo;
 			int smp, to;
+			int dct;
 			int rvv;
 
+			/* Clear note delay before duplicating channels:
+			 * it_note_delay_nna.it */
+			xc->delay = 0;
+
 			note = key + sub->xpo + transp;
 			smp = sub->sid;
-			if (smp >= mod->smp || mod->xxs[smp].len == 0) {
+			if (!IS_VALID_SAMPLE(smp)) {
 				smp = -1;
 			}
+			dct = sub->dct;
 
 			if (not_same_smp) {
 				fix_period(ctx, chn, sub);
-				libxmp_virt_resetchannel(ctx, chn);
+				/* Toneporta, even when not executed, disables
+				 * NNA and DCAs for the current note:
+				 * portamento_nna_sample.it, gxsmp2.it */
+				libxmp_virt_setnna(ctx, chn, XMP_INST_NNA_CUT);
+				dct = XMP_INST_DCT_OFF;
 			}
 			to = libxmp_virt_setpatch(ctx, chn, candidate_ins, smp,
-				note, sub->nna, sub->dct, sub->dca);
+				note, key, sub->nna, dct, sub->dca);
 
 			/* Random value for volume swing */
 			rvv = sub->rvv & 0xff;
 			if (rvv) {
 				CLAMP(rvv, 0, 100);
-				xc->rvv = rand() % (rvv + 1);
+				xc->rvv = libxmp_get_random(&ctx->rng, rvv + 1);
 			} else {
 				xc->rvv = 0;
 			}
@@ -1227,7 +1304,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			rvv = (sub->rvv & 0xff00) >> 8;
 			if (rvv) {
 				CLAMP(rvv, 0, 64);
-				xc->rpv = rand() % (rvv + 1) - (rvv / 2);
+				xc->rpv = libxmp_get_random(&ctx->rng, rvv + 1) - (rvv / 2);
 			} else {
 				xc->rpv = 0;
 			}
@@ -1268,10 +1345,11 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	 * finished (OpenMPT test EnvReset.it). This must take place after
 	 * channel copies in case of NNA (see test/test.it)
 	 * Also if we have envelope in carry mode, check fadeout
+	 * Also, only reset the volume envelope. (it_fade_env_reset_carry.it)
 	 */
 	if (ev.ins && TEST_NOTE(NOTE_ENV_END)) {
 		if (check_fadeout(ctx, xc, candidate_ins)) {
-			reset_envelopes(ctx, xc);
+			reset_envelope_volume(ctx, xc);
 		} else {
 			reset_env = 0;
 		}
@@ -1279,11 +1357,14 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	if (reset_env) {
 		if (ev.note) {
-			RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
+			RESET_NOTE(NOTE_ENV_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
 		}
 		/* Set after copying to new virtual channel (see ambio.it) */
 		xc->fadeout = 0x10000;
 	}
+	if (reset_susloop && ev.note) {
+		RESET_NOTE(NOTE_SAMPLE_RELEASE);
+	}
 
 	/* See OpenMPT wnoteoff.it vs noteoff3.it */
 	if (retrig_ins && not_same_ins) {
@@ -1306,7 +1387,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 			if (TEST_NOTE(NOTE_CUT)) {
 				reset_envelopes(ctx, xc);
-			} else if (!toneporta_offset) {
+			} else if (!toneporta_offset || HAS_QUIRK(QUIRK_PRENV)) {
 				reset_envelopes_carry(ctx, xc);
 			}
 			RESET_NOTE(NOTE_CUT);
@@ -1336,6 +1417,8 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 	if (note >= 0) {
 		xc->note = note;
+	}
+	if (note >= 0 || toneporta_offset) {
 		libxmp_virt_voicepos(ctx, chn, xc->offset.val);
 	}
 
@@ -1420,7 +1503,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 			SET_NOTE(NOTE_END);
 			xc->period = 0;
 			libxmp_virt_resetchannel(ctx, chn);
-		} else if (!is_toneporta && IS_VALID_INSTRUMENT(xc->ins) && is_valid_note(e->note - 1)) {
+		} else if (!is_toneporta && IS_VALID_INSTRUMENT(xc->ins) && IS_VALID_NOTE(e->note - 1)) {
 			struct xmp_instrument *xxi = &mod->xxi[xc->ins];
 
 			xc->key = e->note - 1;
@@ -1445,7 +1528,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 				note = xc->key + sub->xpo + transp;
 				smp = sub->sid;
 
-				if (mod->xxs[smp].len == 0) {
+				if (!IS_VALID_SAMPLE(smp)) {
 					smp = -1;
 				}
 
@@ -1510,7 +1593,7 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	struct xmp_module *mod = &m->mod;
 	struct channel_data *xc = &p->xc_data[chn];
 	struct xmp_subinstrument *sub;
-	int is_smix_ins;
+	struct xmp_instrument *xxi;
 	int ins, note, transp, smp;
 
 	xc->flags = 0;
@@ -1518,33 +1601,39 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	if (!e->ins)
 		return 0;
 
-	is_smix_ins = 0;
 	ins = e->ins - 1;
 	SET(NEW_INS);
-	xc->fadeout = 0x10000;
 	xc->per_flags = 0;
 	xc->offset.val = 0;
-	RESET_NOTE(NOTE_RELEASE);
+	RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
-	xc->ins = ins;
-
-	if (ins >= mod->ins && ins < mod->ins + smix->ins) {
-		is_smix_ins = 1;
-		xc->ins_fade = smix->xxi[xc->ins - mod->ins].rls;
+	xxi = libxmp_get_instrument(ctx, ins);
+	if (xxi != NULL) {
+		xc->ins_fade = xxi->rls;
 	}
+	xc->ins = ins;
 
 	SET(NEW_NOTE);
 
 	if (e->note == XMP_KEY_OFF) {
 		SET_NOTE(NOTE_RELEASE);
 		return 0;
+	} else if (e->note == XMP_KEY_FADE) {
+		SET_NOTE(NOTE_FADEOUT);
+		return 0;
+	} else if (e->note == XMP_KEY_CUT) {
+		SET_NOTE(NOTE_END);
+		xc->period = 0;
+		libxmp_virt_resetchannel(ctx, chn);
+		return 0;
 	}
 
 	xc->key = e->note - 1;
+	xc->fadeout = 0x10000;
 	RESET_NOTE(NOTE_END);
 
-	if (is_smix_ins) {
-		sub = &smix->xxi[xc->ins - mod->ins].sub[0];
+	if (ins >= mod->ins && ins < mod->ins + smix->ins) {
+		sub = &xxi->sub[0];
 		if (sub == NULL) {
 			return 0;
 		}
@@ -1559,15 +1648,15 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 			xc->smp = smp;
 		}
 	} else {
-		sub = is_valid_note(xc->key) ?
+		sub = IS_VALID_NOTE(xc->key) ?
 			get_subinstrument(ctx, xc->ins, xc->key) : NULL;
 		if (sub == NULL) {
 			return 0;
 		}
-		transp = mod->xxi[xc->ins].map[xc->key].xpo;
+		transp = xxi->map[xc->key].xpo;
 		note = xc->key + sub->xpo + transp;
 		smp = sub->sid;
-		if (mod->xxs[smp].len == 0)
+		if (!IS_VALID_SAMPLE(smp))
 			smp = -1;
 		if (smp >= 0 && smp < mod->smp) {
 			set_patch(ctx, chn, xc->ins, smp, note);
